function eig_comp = inverse_iteration(A,B,delta,eta,cur_shift,eigs_tol)
% auxiliary function called by perform_test_hybrid
% call : inverse_iteration(AK,cur_shift)
% task : apply shifted inverse iteration to AK
%        (the matrix with Kronecker product blocks)
%        with shift cur_shift which is equivalent
%        to applying power iteration to the matrix
%        (AK-cur_shift I)^-1. The matrix vector
%        multiplication (AK-cur_shift I)^-1 u is
%        achieved by solving a Sylvester equation
%        depending on A, B, delta, eta and cur_shift.
%
%
% Input:
%           A           : The state coefficient matrix.
%           B           : The input coefficient matrix.
%           delta       : Scalar depending on current upper
%                         and lower bounds for tau(A,B).
%           eta         : Scalar depending on current upper
%                         and lower bounds for tau(A,B).
%           cur_shift   : Shift applied to AK.
%           eigs_tol    : Tolerance used by eigs for termination.
%
% Output:
%           inv_eig     : Largest eigenvalue of the matrix 
%                         (AK-cur_shift*I)^-1.

[sar,sac] = size(A);
[sbr,sbc] = size(B);


if sar ~= sac
    error('inverse_iteration: input matrix A must be square')
end

if sbr < sbc
    error('inverse_iteration: input matrix must have at least as many rows as its columns')
end
                            

% block matrices appearing in the Sylvester equation
eyen = eye(sar);
Bhat = B*B'/delta - delta*eyen;
eyen2 = eye(sar*sar);
zerosn = zeros(sar);
zerosn2 = zeros(sar*sar);    


% set the option fields 
% issym : 0:matrix is nonsymmetric, 1:matrix is symmetric
options.issym = 0;
options.v0 = ones(2*sar*sar,1);

% isreal : 0:matrix is complex, 1:matrix is real
options.isreal = isreal(A) & isreal(Bhat);
% tol : termination criterion
options.tol = eigs_tol;
options.disp = 0;
options.maxit = 40000;


% The matrices appearing in the Sylvester equation
% MX+XN=U which needs to be solved to compute the product
% (AK-cur_shift*I)^-1 u. M and N are input arguments to
% inverse_times_vector solving the Sylvester equation.
% U depends on the vector u and created inside the function
% inverse_times_vector.
M = [A'-cur_shift*eyen  delta*eyen; ...
                  Bhat  -A+cur_shift*eyen];
N = [A-(cur_shift+eta)*eyen  Bhat; ...
                 delta*eyen  -A'+(cur_shift+eta)*eyen];
                              


[v,d] = eigs(@inverse_times_vector, 2*sar*sar, 1, ...
                                cur_shift, options, M, N);
                                


% v is the eigenvector corresponding to the eigenvalue closest
% to cur_shift
eig_comp = d(1,1);

% keyboard

return;