function [beta, w] = cdi(A, prtlevel,keyb)
% Michael Overton and Emre Mengi (Last updated on March 11th 2005)
%
% call: function [beta, w] = cdi(A, prtlevel, keyb)
%
%  DEFINITION:
%  The discrete distance to instability of the matrix A is the spectral
%  norm of the smallest perturbation such that the resulting matrix
%  has an eigenvalue with non-negative real part, i.e.
%    (1) beta(A) = inf{||E|| : det(A+E-zI)=0 for some z with Re(z) >= 0}.
%  An equivalent definition in terms of singular values is
%    (2) beta(A) = min_{w} sigma_min (A - wiI)
%  where sigma_min denotes minimum singular value.
%
%  METHOD:
%  Quadratically convergent Boyd-Balakrishnan method based on
%  definition (2) is implemented. To minimize the function 
%  f(w)=sigma_min (A - wiI) the quadratically convergent 
%  method introduced in [1] is used. At each iteration given
%  an estimate alpha and a set containing the midpoints of the intervals
%  in which the inequality f(w) <= alpha is satisfied, the function 
%  f(w) is computed at each midpoint. The estimate is updated
%  to the minimum of these values. Finally all w satisfying alpha=f(w)
%  are found. Using these w values the set of midpoints are updated.
%
%  To find w satisfying alpha = f(w), following slightly modified 
%  version of theorem 1 in [2] is used.
%  Theorem : The matrix A-wiI has alpha as one of its singular values
%            if and only if H(alpha) has the imaginary eigenvalue wi,
%            where H(alpha) = [A -alpha*I; alpha*I -A']
%
%  References:
%  1- S.Boyd and V.Balakrishnan. A regularity result for the singular
%     values of a transfer matrix and a quadratically convergent 
%     algorithm for computing its L_{inf}-norm. Systems and Control
%     Letters, 15:1-7, 1990.
%  2- R.Byers. A bisection method for measuring the distance of a 
%     stable matrix to the unstable matrices. SIAM Journal on
%     Scientific and Statistical Computing, 9:875-881, 1988.
%
%
%
% Note     : In this implementation at each iteration the function
%            f(w) is computed at the midpoint of each consecutive
%            pair such that f(w)=alpha holds. If we have a reliable
%            and efficient procedure to identify the multiplicities 
%            of the zeros of f(w)-alpha, we may as well restrict the 
%            function evaluations to the midpoints of the intervals 
%            in which the function f is less than the estimate alpha.
%
%
% Input:
%       A         : Input matrix (must be square)
%       prtlevel  : 1: Print the details at each iteration
%                   0: Do not print(Default).
%       keyb      : 1: Interact with the user
%                   0: Do not interact(Default).
%
% Output:
%       beta      : Distance to instability for the input matrix A.
%       w         : The point where beta(A) is attained, i.e.
%                   beta(A) = min_{w} f(w).


A = full(A);

if nargin <= 2
    keyb = 0; % default: don't interact with the user
end

if nargin <= 1
    prtlevel = 0; % default: don't print
end




% error checks
if isnaninf(A)|isnaninf([prtlevel keyb])
    error('cdi: nan or inf arguments not allowed')
end

n = length(A);
if size(A) ~= [n,n]
    error('cdi: A must be square')
end



% check whether A is stable or not. If it is not stable return 0
if max(real(eig(A))) >= 0
    beta = 0;
    w = 0;
   
    return;
end


% initializations
wmid = [0];
% arbitrary values for the first iteration
alphaold = 0;
alpha = -.1;
wbestt = [];
wbest = wmid;


fail_horizontal = 0;

iter = 0;
realtol = 10^-8*norm(A);


% main loop
%while abs(alpha - alphaold) > 10^(-4) & ~fail_horizontal
while alpha < alphaold & ~fail_horizontal
    iter = iter + 1;

    % this is a quadratically convergent method, the number of
    % iterations is supposed to be at most 6,7.
    if (iter > 20)
        error('cdi: too many iterations')
    end

    wbestt = wbest;
    alphaold = alpha;
    % compute the function value at each midpoint, return
    % the minimum of these function values and the midpoint
    % at which the minimum function value is attained
    [alpha,wbest] = cdi_alpha(A,wmid);
    
    if prtlevel > 0
        fprintf('cdi: wbest = %22.15e  \n', wbest);
    end
        
    

    % at the first iteration alpha values are arbitrary
    % update them so that the program enters to the if statement below
    if (iter == 1)
        alphaold = alpha + .1;
    end

	

    % if we have a better estimate compared to the previous iteration
    if (alpha < alphaold)
        wmidold = wmid;

        % find the zeros of the function g(w) = f(w)-alpha
        % return the midpoints of the consecutive pair of zeros
        wmid = cdi_w(A,alpha,wbest,realtol,prtlevel);



        if keyb ~= 0
            keyboard
        end
        

        % if wmid returned by cdi_w is empty, we
        % reached machine accuracy, current alpha and wbest
        % are the optimum values and the location where the
        % optimum value is attained.
        fail_horizontal = isempty(wmid);

        
    else
    % Otherwise the estimate in the previous iteration is better,
    % so we should have reached machine accuracy in the previous
    % iteration. Restore the values from the previous iteration.
        wbest = wbestt;
        alpha = alphaold;
    end
    
    if prtlevel > 0 & iter > 1
        fprintf('cdi: iteration %d  \n', iter);
        fprintf('cdi: alpha = %22.15e  \n', alpha);
        fprintf('cdi: wmid = %22.15e  \n', wmid);    
    end
    
end % end of main loop, while alpha < alphaold ...



if isempty(wbest)

    error('cdi: Failed in the first iteration')
end



% set the return variables
beta = alpha;
w = wbest;

return;