function wmid = cdi_w(A,alpha,wwant,realtol,prtlevel)
% Michael Overton and Emre Mengi (Last updated on March 11 2005)
%
% called by cdi.m
%
% Given an estimate alpha of the distance to instability find w
% values satisfying
%       (*)     alpha=f(w)=sigma_min(A-wiI) 
% where sigma_min denotes minimum singular value. Let the vector w'=[w(1),w(2),...w(m)] 
% contains the values satisfying (*), the vector wmid containing the midpoints of the 
% consecutive pairs of w' is returned. Formally wmid = [wmid(1),wmid(2),...,wmid(m-1)]
% where for all i, wmid(i) = (w(i) + w(i+1)) / 2. 
%
% Input:
%       A       : Input matrix(must be square).
%       alpha   : Current estimate for the distance to instability.
%       wwant   : The location where the current estimate value is
%                 attained.
%       realtol : A tolerance value to accept whether the real part of an
%                 eigenvalue of H(alpha) is zero or not (See the comments 
%                 at the beginning of cdi.m for the definition
%                 of H(alpha)).
%       prtlevel: 1: print the details.
%                 0: do not print.
%       h       : Handle for the waitbar.
%
% Output:
%       wmid    : Vector of midpoints, i.e. wmid(i) = (w(i) + w(i+1)) / 2
%                 for 1 <= i <= m-1 where w(1),w(2),...,w(m) are w values
%                 satisfying alpha=f(w).


% get the size of the input matrix and check whether it's square or not.
n = length(A);
if size(A) ~= [n,n]
    error('A must be square')
end


% for the inputs alpha and A create H(alpha) and compute the eigenvalues of
% H(alpha).
In = eye(n);


H = [A -alpha*In; alpha*In -A'];
eH = eig(H);




% extract the eigenvalues whose real parts are close enough to 0 (by the
% real parts close enough to 0 we mean the real parts smaller than the 
% tolerance value).
ind = find(abs(real(eH)) <= realtol);
eH = eH(ind);




if (isempty(eH)) % check if eH contains a pure imaginary eigenvalue 
    wmid = [];
else

    % sort eH according to the imaginary parts
    w = sort(imag(eH));


    ind2 = [];
    % for each j check if alpha is the smallest singular value of A-wiI
    % previous step guaranteed that alpha is a singular value of A-wiI 
    for j = 1:length(w)
        s = svd(A - w(j)*i*In);
        s = sort(s);
    

        [closes,minind] = min(abs(s-alpha));
        if minind == 1
            ind2 = [ind2; j];
        end
    end


    removed = length(w) - length(ind2);

    % remove the w values for which the matrix A-wiI has the singular value
    % alpha but not the smallest one.
    if removed > 0
        if prtlevel > 0
            fprintf('cdi_w: singular value test removed %d eigenvalues \n', removed);
        end

        w = w(ind2);       
    end
    
    
    % all of the eigenvalues shouldn't be removed by the singular value
    % test. If the matrix A-wiI has the singular value alpha, then
    % sigma_min(A-wiI) <= alpha. We also know that sigma_min is a 
    % continuous function of w. Thus if the vector w is not empty before
    % the singular value test, it shouldn't be afterwards. 
    if (isempty(w))
        warning('singular value test removed all of the intersection points')
        wmid = [];
        
        return;
    end


    % Furthermore at this point there should be at least two eignvalues, since 
    % f(w) goes to infinity as w goes to inifinity or -infinity.   
    if (mod(w,2) == 1)
        error('after singular value test there are odd number of eigenvalues')
    end
	
    
    
    
    
    
    
    
    % organize in pairs and take midpoints
    ind = 0;

    for j=1:length(w)-1
        wlow = w(j);
        whigh = w(j+1);


        % before taking the midpoint, if this interval is not very short,
        % check and see if wwant is in this interval, well away from the
        % end points.  If so, break this pair into two pairs, one above
        % and one below wwant, and take midpoints of both.
        inttol = .01 * (whigh - wlow);
        if wwant > wlow + inttol & ...
                wwant < whigh - inttol

            % lower midpoint
            ind = ind + 1;
            wmid(ind,1) = (wlow + wwant)/2;
			
            % upper midpoint
            ind = ind + 1;
            wmid(ind,1) = (wwant + whigh)/2;
        else

            % otherwise, if wwant is not in the interval
            % take the midpoint of wlow and whigh
            ind = ind + 1;
            wmid(ind,1) = (wlow + whigh)/2;
        end
    end % end of for j= ...

    
    
end % end of else

return;
