function thetamid = ddi_theta(A,alpha,thetawant,radtol,prtlevel)
% Emre Mengi and Michael Overton (Last updated on March 16 2005)
%
% called by ddi.m
%
% Given an estimate alpha for the discrete distance to instability find
% theta values satisfying
%
%       (*)     alpha=g(theta) 
%
% where g(theta)=svd_{min}(A - e^(i*theta)*I). Let the vector 
% theta'=[theta(1),theta(2),..theta(m)] contain the values satisfying (*) in
% increasing order, then the vector thetamid filled with the midpoints of 
% the consecutive pairs in theta' is returned. Formally 
% thetamid = [thetamid(1),thetamid(2),...,thetamid(m)] where 
%
%         thetamid(i) = (theta(i) + theta(i+1)) / 2           i < m 
%                       (theta(1) + theta(m) + 2*pi) / 2      i = m
%
% Input:
%       A           : Input matrix (must be square).
%       alpha       : Current estimate for the discrete distance to instability.
%       thetawant   : The location where the previous estimate value is
%                     attained.
%       radtol      : A tolerance value to accept whether the modulus of a
%                     generalized eigenvalue of the pencil F(alpha)-lambda*G(alpha)
%                     is one or not.(See the comments at the beginning of 
%                     ddi.m for the definition of F(alpha) and
%                     G(alpha).
%       prtlevel    : 1: print the details.
%                     0: do not print.
%
% Output:
%       thetamid    : Vector of midangles, i.e. thetamid(i) = (theta(i) + theta(i+1)) / 2
%                     for 1 <= i <= m-1 and thetamid(m) = (theta(1) + theta(m) + 2*pi) / 2 
%                     where theta(1),theta(2),...,theta(m) are theta values
%                     satisfying alpha=g(theta).



% get the size of the input matrix and check whether it's square or not.
n = length(A);
if size(A) ~= [n n]
    error('A must be square')
end


% for the inputs alpha and A create F(alpha), G(alpha) and compute the generalized 
% eigenvalues of the pencil F(alpha)-lambda*G(alpha)
In = eye(n);
On = zeros(n);
F = [-alpha*In A; In On];
G = [On In; A' -alpha*In];
eFG = eig(F,G);



% extract the eigenvalues whose moduli are close enough to 1(by the
% moduli close enough to 1 we mean the moduli which are apart from 
% 1 at most by the tolerance value).
ind = find(abs(abs(eFG)-1) < radtol);
eFG = eFG(ind);



if (isempty(eFG)) % check if eFG contains an eigenvalue with modulus 1.
    thetamid = [];
else
    % sort eFG according to the angle of the components
    theta = sort(angle(eFG));


    ind2 = [];
    % for each j check if alpha is the smallest singular value of A-e^{i*theta}*I
    % previous step guaranteed that alpha is a singular value of A-e^{i*theta}*I
    for j = 1:length(theta)
        if (theta(j) < 0)
            theta(j) = theta(j)+2*pi;
        end
        
        s = svd(A - exp(i*theta(j))*In);
        s = sort(s);
    

        [closes,minind] = min(abs(s-alpha));
        
        if minind == 1
            ind2 = [ind2; j];
        end
    end


    removed = length(theta) - length(ind2);

    % remove the theta values for which the matrix A-e^{i*theta}*I has the singular value
    % alpha but not the smallest one.
    if removed > 0
        if prtlevel > 0
            fprintf('ddi_theta: singular value test removed %d eigenvalues \n', removed);
        end

        theta = theta(ind2);
    end




    % all of the eigenvalues shouldn't be removed by the singular value
    % test. If the matrix A-e^{i*theta}*I has the singular value alpha, then
    % sigma_min(A-e^{i*theta)*I) <= alpha. We also know that sigma_min is a 
    % continuous function of theta and let beta = max_{theta} sigma_min(A-e^{i*theta}*I), 
    % beta >= alpha. Thus if the vector theta s not empty before the singular
    % value test, it shouldn't be afterwards.
    if (isempty(theta))
        warning('singular value test removed all of the intersection points')
        thetamid = [];
        
        return;
    end
    
    
    
    theta = sort(theta);
    
    % organize in pairs and take midpoints
    ind = 0;
    
    
    % shift thetawant, the angle from the previous iteration, into
    % the interval [0,2pi]
    if (thetawant < 0)
        thetawant = thetawant + 2*pi;
    end


    for j=1:length(theta)
        thetalow = theta(j);


        if (j < length(theta))
            thetahigh = theta(j+1);
        else
            % the last interval wraps around
            thetahigh = theta(1)+2*pi;
        end



        % before taking the midpoint, if this interval is not very short,
        % check and see if thetawant is in this interval, well away from the
        % end points.  If so, break this pair into two pairs, one above
        % and one below thetawant, and take midpoints of both.
        inttol = .01 * (thetahigh - thetalow);

        if (thetawant+2*pi > thetalow + inttol & ...
                thetawant+2*pi < thetahigh - inttol)
            thetawantt = thetawant + 2*pi;
        else
            thetawantt = thetawant;
        end


        if thetawantt > thetalow + inttol & ...
                thetawantt < thetahigh - inttol

            % lower midpoint
            ind = ind + 1;
            thetamid(ind,1) = (thetalow + thetawantt)/2;

            % shift thetamid(ind) into the interval [-pi,pi] again
            if (thetamid(ind,1) >= 2*pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
            if (thetamid(ind,1) >= pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end		
	
		
            % upper midpoint
            ind = ind + 1;
            thetamid(ind,1) = (thetawantt + thetahigh)/2;

            % shift thetamid(ind) into the interval [-pi,pi] again
            if (thetamid(ind,1) >= 2*pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
            if (thetamid(ind,1) >= pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
        else


            % otherwise, if thetawant is not in the interval
            % take the midpoint of thetalow and thetahigh
            ind = ind + 1;
            thetamid(ind,1) = (thetalow + thetahigh)/2;

            % shift thetamid(ind) into the interval [-pi,pi] again
            if (thetamid(ind,1) >= 2*pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
            if (thetamid(ind,1) >= pi)
                thetamid(ind,1) = thetamid(ind,1) - 2*pi;
            end
        end % end of else
        
    end % end of for

	

end % end of else (isempty(eFG))

return;