function thetamid = numr_theta(A,f,thetawant,radtol,prtlevel,plotnum)
% Michael Overton and Emre Mengi (Last updated on March 9th 2005)
%
% called by numr.m
%
% Given an estimate f for the numerical radius find theta
% values satisfying 
%       (*)     f=g(theta) 
% where g(theta) = lambda_{max}(H(theta)). Let the vector theta'=[theta(1),theta(2),...theta(m)]
% contain the values satisfying (*) in sorted order. The vector thetamid containing 
% the midpoints of the consecutive points in theta' is returned. Formally 
% thetamid = [thetamid(1),thetamid(2),...,thetamid(m)] where 
%
%           thetamid(i) = (theta(i) + theta(i+1)) / 2           if i<m
%                         (theta(1) + theta(m) + 2*pi)/2        if i=m
%
% Input:
%       A           : Input matrix (must be square).
%       f           : Current estimate for the numerical radius.
%       thetawant   : The location where the current estimate value is
%                     attained.
%       radtol      : A tolerance value to accept whether the modulus of a generalized
%                     eigenvalue of R(f)-lambda*S is zero or not (See the comments 
%                     at the beginning of dist_inst_2way.m for the definition
%                     of R(f) and S).
%       prtlevel    : 1: print the details.
%                     0: do not print.
%       h           : Handle for the waitbar.
%       plotnum     : Figure number.
%
% Output:
%       thetamid    : Vector of midpoints, i.e. thetamid(i) = (theta(i) + theta(i+1)) / 2
%                     for 1 <= i <= m-1 and thetamid(m) = (theta(1) + theta(m) +2*pi)/2 
%                     where theta(1),theta(2),...,theta(m) are theta values
%                     satisfying f=g(theta).


% get the size of the input matrix and check whether it's square or not.
n = length(A);
if size(A) ~= [n n]
   error('numr_theta: A must be square')
end


% for the inputs f and A create R(f),S and compute the eigenvalues of
% the pencil R(f)-lambda*S
R = [2*f*eye(n) -A';eye(n) zeros(n)];
S = [A zeros(n);zeros(n) eye(n)];
eRS = eig(R,S);


% extract the eigenvalues whose modulus are close enough to 1 (by the
% modulus close enough to 1 we mean the modulus which is apart from 
% 1 at most by the tolerance value).
ind = find((abs(eRS) < (1 + radtol)) & (abs(eRS) > (1 - radtol)));
eRS = eRS(ind);





if (isempty(eRS)) % check if eRS contains an eigenvalue with magnitude 1
   	thetamid = [];
else

	% sort eRS wrt thetamid values
 	theta = sort(angle(eRS));


	ind2 = [];
    
    
	% for each j check if f is the largest eigenvalue of H(theta(j))
	% previous step guaranteed that f is an eigenvalue of H(theta(j))
	for j = 1:length(theta)
        if (theta(j) < 0)
            theta(j) = theta(j)+2*pi;
        end
        
		H = 0.5*((cos(theta(j)) + i*sin(theta(j)))*A + ...
				(cos(theta(j)) - i*sin(theta(j)))*A');
		eH = eig(H);
		eH = sort(eH);


		[closeig,minind] = min(abs(eH-f));
		if minind == n
			ind2 = [ind2; j];
		end
	end
    


	removed = length(theta) - length(ind2);    

    % remove the theta values for which the matrix H(theta) has the
    % eigenvalue f but not the largest one.
    if removed > 0
        if prtlevel > 0
            fprintf('numr_theta: eigenvalue test removed %d eigenvalues \n', removed);
        end

        theta = theta(ind2);
	end
    



    % all of the eigenvalues shouldn't be removed by the eigenvalue test.
    % If the matrix H(theta) has the eigenvalue f, then lambda_{max}(H(theta)) >= f.
    % We also know that lambda_{max} is a continuous function of theta and
    % let beta = min_{theta} lambda_{max}(H(theta)), beta <= f. Thus if the 
    % vector theta is not empty before the eigenvalue test, it shouldn't be
    % afterwards.
   	if (isempty(theta))

       		warning('eigenvalue value test removed all of the intersection points')
            warning('the result may be inaccurate')
            thetamid = [];
            
            return;
	end


    theta = sort(theta);
    
	% organize in pairs and take midpoints
 	ind = 0;

 	% shift thetawant, the angle from the previous iteration, into
 	% the interval [0,2pi]
 	if (thetawant < 0)
     		thetawant = thetawant + 2*pi;
 	end


    for j=1:length(theta)
        thetalow = theta(j);

		if (j < length(theta))
            thetahigh = theta(j+1);
		else
            % the last interval wraps around
            thetahigh = theta(1) + 2*pi;
		end

        
                    

		% before taking the midpoint, if this interval is not very short,
		% check and see if thetawant is in this interval, well away from the
		% end points.  If so, break this pair into two pairs, one above
		% and one below thetawant, and take midpoints of both.
		inttol = .01 * (thetahigh - thetalow);
        
        % this is needed for the last interval
        if (thetawant+2*pi > thetalow + inttol & ...
          		thetawant+2*pi < thetahigh - inttol)
            thetawantt = thetawant + 2*pi;
        else
            thetawantt = thetawant;
        end
        
        
        if thetawantt > thetalow + inttol & ...
                thetawantt < thetahigh - inttol
            % lower midpoint
   		    ind = ind + 1;
   		    thetamid(ind,1) = (thetalow + thetawantt)/2;

   		    % shift thetamid(ind) into the interval [-pi,pi] again
   		    if (thetamid(ind,1) >= 2*pi)
       			thetamid(ind,1) = thetamid(ind,1) - 2*pi;
   		    end
   		    if (thetamid(ind,1) >= pi)
       			thetamid(ind,1) = thetamid(ind,1) - 2*pi;
   		    end
            			
		    % upper midpoint
		    ind = ind + 1;
		    thetamid(ind,1) = (thetawantt + thetahigh)/2;

		    % shift thetamid(ind) into the interval [-pi,pi] again
		    if (thetamid(ind,1) >= 2*pi)
    			thetamid(ind,1) = thetamid(ind,1) - 2*pi;
		    end
		    if (thetamid(ind,1) >= pi)
    			thetamid(ind,1) = thetamid(ind,1) - 2*pi;
		    end
            
        else


		    % otherwise, if thetawant is not in the interval
		    % take the midpoint of thetalow and thetahigh
            ind = ind + 1;
		    thetamid(ind,1) = (thetalow + thetahigh)/2;

		    % shift thetamid(ind) into the interval [-pi,pi] again
		    if (thetamid(ind,1) >= 2*pi)
		        thetamid(ind,1) = thetamid(ind,1) - 2*pi;
		    end
		    if (thetamid(ind,1) >= pi)
       			thetamid(ind,1) = thetamid(ind,1) - 2*pi;
   		    end
            
        end % end of else
        
    end % end of for

    if plotnum > 0
        figure(plotnum);
        plot([0 2*pi], [f f], 'm-')
        for j = 1:length(theta)         
            if theta(j) < 0
                theta(j) = theta(j) + 2*pi;
            end
            
            plot(theta(j),f,'c*')
        end
    end

end  % end of else (isempty(eRS))

return;