function [f, z] = pspr(A, epsln, prtlevel, plotfig, keyb)
% Michael Overton and Emre Mengi (Last updated on March 15 2005)
%
% call: function [f, z] = pspr(A, epsln, prtlevel, plotfig, keyb)
% task
%  Quadratically convergent criss-cross method to compute
%  the eps-pseudospectral radius of A.
%
%  DEFINITION:
%  The epsln-pseudospectral radius of A is the globally optimal value of: 
%      max |z|
%      s.t. sigma_min(A - z I) = epsln     (smallest singular value)
%
%  DETAILS OF THE METHOD:
%  This is an implementation of the criss-cross method described in [1].
%  At each iteration the method refines an estimate of the pseudospectral 
%  radius r and a set of midpoints of the intervals in which the circle of 
%  radius r lies inside the pseudospectrum. Initially the estimate is the
%  spectral radius. At each iteration first a radial search from each
%  midpoint returns the largest modulus intersection point of the pseudospectrum 
%  with the ray from the origin going through the midpoint. The estimate
%  is updated to the largest of these values. By the help of a circular
%  search the midpoints of the intervals in which the circle with 
%  radius equal to the new estimate lies inside the pseudospectrum are 
%  computed. The radial search and the circular search are performed by solving 
%  a Hamiltonian eigenvalue problem and by solving a symplectic generlized 
%  eigenvalue problem, respectively (see [1] for the details).
%
%
%  References
%  1- E.Mengi and M.L.Overton. Algorithms for the computation of the
%     pseudospectral radius and the numerical radius of a matrix. IMA
%     Journal of Numerical Analysis. To appear.
%
%
%  input
%     A         square matrix
%     epsln     real >= 0 (0 implies f is spectral radius)
%     prtlevel  1: chatty; 0: prints only if problems arise (default)
%     plotfig   0: no plot, otherwise figure number for plot
%     keyb      0: no keyboard mode(default), otherwise after each 
%		           iteration enter keyboard mode
%              
%  output
%     f         the epsln-pseudospectral radius of A
%     z         one of the global maximizers except when
%               the matrix is real. If the input matrix is real 
%               it contains either a global optimizer on the
%               real axis or a complex conjugate pair.

A = full(A);

if nargin <= 4
    keyb = 0;     % default: don't interact with the user
end
    
if nargin <= 3
    plotfig = 0; % default: don't draw
end

if nargin <= 2
    prtlevel = 0; % default: don't print
end



% error checks
if isnaninf(A)|isnaninf([epsln prtlevel plotfig keyb])
   error('pspr: nan or inf arguments not allowed')
end

if epsln < 0 | imag(epsln) ~= 0
   error('pspr: epsln must be nonnegative real')
end

n = length(A);
if size(A) ~= [n n]
   error('pspr: A must be square')
end




% draw the eigenvalues of A
eA = eig(A);
if plotfig > 0
   figure(plotfig);
   plot(real(eA), imag(eA), 'ko')
   hold on
end



smalltol = 1e-8*max(norm(A),epsln);

if epsln == 0  % pseudospectrum is just the spectrum
   [sortreal, indx] = sort(-abs(eA));
   eA = eA(indx);
   
   % return pure spectral radius
   f = abs(eA(1));

   % and the eigenvalues with magnitudes equal to the spectral radius
   ind = find(abs(eA) >= f - smalltol);
   z = eA(ind);
   
   
else
    if (epsln < 10^-9)
        warning('epsilon is too small, the result may not be accurate enough')
    end

   rold = -0.00001;
   % initially r is the spectral-radius
   [r,ind] = max(abs(eA));  % initial iterate
   
   if prtlevel > 0
       fprintf('pspr: initially  \n');
       fprintf('pspr: r = %22.15e  \n', r);
   end


   % theta is the angle of an eigenvalue z, s.t |z| = r
   theta = angle(eA(ind));
   thetaold = theta;

   thetabest = [];
   

   
   iter = 0;
   no_thetaeig = 0;
   mE = -epsln*eye(n);
   
   % realtol is used to detect zero real parts up to some tolerance.
   realtol = smalltol;
   % radtol is the tolerance determining how far the eigenvalue magnitudes
   % can be apart from 1 (used by the circular search).
   radtol = smalltol;
   

   while ~no_thetaeig & r > rold    % r is increasing in exact arithmetic

      
      iter = iter + 1;
      
      
      if iter > 20
          error('pspr: too many steps')
      end
      

      thetabestt = thetabest;
      rold = r;

      % given the resulting directions in theta(computed in the
      % previous iteration except when iter=1), look for the circle with
      % the greatest radius intersecting pseudo-spectrum boundary at the
      % given directions.
      % note: input theta is a vector, but output r is a scalar: the max
      % of the max values.  thetabest is the corresponding theta value and is
      % a scalar, even if there was a tie e.g. from a complex conjugate pair.
      [r, thetabest] = pspr_rad(A, mE, theta, realtol, plotfig, rold, iter);

      % Ensure sigma_min(A-r e^(i theta_rho)I) > epsilon at the first interation
      % where r is the estimate of the pseudospectral radius at the first
      % iteration and theta_rho is the initial radial search direction.
      if (iter == 1)
          [u,e,v] = svd(A-r*exp(i*thetabest)*eye(n));
          mins = e(n,n);
          ds = real(v(n,:)*u(:,n));
          if (ds < sqrt(smalltol))
              ds = sqrt(smalltol);
          end
          beta = epsln - mins;
          incr = abs(beta/ds);
          initer = 0;
          while (mins <= epsln)
              initer = initer + 1;
              r = r + incr;
              mins = svd(A-r*exp(i*thetabest)*eye(n));
          end
       
      end
      
      
      if (keyb ~= 0)
           keyboard
      end

      

      
      if r > rold
      	% given current r, look for all relevant intersections of the circle
      	% with radius r with the pseudospectrum, process and return pair midpoints.
      	% note: input r is a scalar, but output theta is a vector.
      	% the input thetabest is a scalar.
      	% there is one difference compared to pseudospectral absicca case
      	% let theta contains t1 <= t2 <= ...  <= tn
      	% the intervals are circular that is we have to check the midpoints
      	% of [t1,t2], [t2,t3], ..., [tn-1,tn],[tn,t1].
      	% In one of these directions there should be a point on the
      	% boundary with a greater radius compared to current maximum radius
        thetaold = theta;
        theta = pspr_theta(A, mE, epsln, r, thetabest, iter, radtol, ...
          	 smalltol, plotfig, prtlevel);

        if keyb ~= 0
            keyboard
        end
      
      	no_thetaeig = isempty(theta);
        
        if (no_thetaeig)
            rold = r;
        end
            
      else
          thetabest = thetabestt;
          r = rold;
      end % end of else
      
      
      if prtlevel > 0
          fprintf('pspr: iteration %d  \n', iter);
          fprintf('pspr: r = %22.15e  \n', r);
          fprintf('pspr: theta= %22.15e  \n', theta);
      end
      
            
   end % end of while

   
      
   
   if isempty(thetabest)
       error('Failed in the first iteration (please set epsln larger)');
   end
   
   
   
   f = r;
   z = r*(cos(thetabest) + i*sin(thetabest));
   
   if isreal(A) & ~isreal(z)
       z = [z; z'];
   end
   
                
end % end of else


return;