function [f, z] = poly_pspa(P, epsln, gamma, prtlevel, plotfig, keyb, eigsolver)
% Emre Mengi and Michael Overton
% (Last updated on June 30 2006)
%
% call:  [f, z] = poly_pspa(P, epsln, gamma, prtlevel, plotfig, keyb, eigsolver)
%  Quadratically convergent criss-cross method to compute
%  the eps-pseudospectral abscissa of P.
%  
% DEFINITION:
%   The epsln-pseudospectral abscissa of P is the globally optimal value of: 
%      max Re z
%      s.t. sigma_min(P(z))/p_gamma(|z|) = epsln     (smallest singular value)
%
% where
%       p_gamma(x) = sqrt(sum_j=0^k x^(2j) gamma_j^2)
%
% METHOD:
% The extension of the method in [1] for matrix polynomials is implemented. 
% At each iteration given an estimate alpha first a vertical search finds the 
% intersection points of the vertical line y=alpha and the polynomial 
% epsln-pseudospetrum boundary. From the intersection points segments on the 
% line lying inside epsln-pseudospectrum are determined. From the midpoint of 
% each segment a horizontal search returns the rightmost intersection point of 
% the pseudospectrum boundary and the horizontal line passing through the midpoint. 
% The estimate alpha is refined to the largest value returned by the horizontal 
% searches. Horizontal and vertical searches can be performed by solving
% *-even eigenvalue problems (for details see [2]).
%
% note: In windows user has the flexibility to choose the Hamiltonian
%       eigenvalue solver by setting the parameter eigsolver appropriately.
%       In all other platforms set eigsolver = 0, or don't specify it so
%       that Matlab's eig is used.
%
%  input
%     P         square matrix polynomial
%     epsln     real >= 0 (0 implies f is spectral abscissa)
%     gamma     vector of scaling factors
%     prtlevel  1: chatty; 0: prints only if problems arise (default)
%     plotfig   0: no plot, otherwise figure number for plot
%     keyb      0: no keyboard mode(default), otherwise after each 
%		           iteration enter keyboard mode
%     eigsolver Hamiltonian eigenvalue solver to be used.
%               0: Matlab's eig (tolerances are needed to accept an 
%                  eigenvalue imaginary; Default)
%               1: The eigenvalue symmetry of *-even polynomials are
%                  preserved, tolerances are not needed (yet to be
%                  implemented).
%              
%  output
%     f         the epsln-pseudospectral abscissa of A
%     z         one of the global maximizers except when
%               the matrix is real. If the input matrix is real 
%               it contains either a global optimizer on the
%               real axis or a complex conjugate pair.
%   
%
%    [1] J.V. Burke, A.S. Lewis and M.L. Overton. Robust stability 
%        and a criss-cross algorithm for pseudospectra. IMA Journal
%        of Numerical Analysis, 23:359-375,2003.
%    [2] E. Mengi. Measures for Robust Stability and Controllability.
%        Ph.D. Thesis. Courant Institute of Mathematical Sciences. New
%        York, August 2006. In preperation.


if nargin <= 6
    eigsolver = 0;
end

if nargin <= 5
    keyb = 0; % default: no user intervention
end

if nargin <= 4 
   plotfig = 0;     % default: no plot
end

if nargin <= 3 
   prtlevel = 0;    % default: no printing
end

if nargin < 2
    error('poly_pspa: polynomial and epsln must be provided as input')
end




if poly_isnaninf(P) | poly_isnaninf([epsln prtlevel plotfig keyb eigsolver])
   error('poly_pspa: nan or inf arguments not allowed')
end
if epsln < 0 | imag(epsln) ~= 0    
   error('poly_pspa: epsln must be nonnegative real')
end


[nr,nc,kp] = size(P);
if nr ~= nc
   error('poly_pspa: P must be square')
end


% gamma if provided by user must consist of nonnegative real entries
% not all zero
if nargin >= 3
    [gr,gc] = size(gamma);
    
    if gr ~= 1 | gc ~= kp
        error('poly_pspa: gamma must be row vector of size k')
    end
            
    if sum(gamma-abs(gamma)) ~= 0
        error('poly_pspa: gamma must consist of nonnegative real numbers')
    end
    
    if gamma*gamma' == 0
        error('gamma must be nonzero')
    end
end
    

n = nr;
k = kp-1;
% gamma is not provided, set it to the default vector [1 1 ... 1]
if nargin == 2
    gamma = ones(1,k+1);
end

if poly_isnaninf(gamma)
   error('poly_pspr: nan or inf arguments not allowed')
end



% draw the eigenvalues of P
eP = mypolyeig(P);
if plotfig > 0
   figure(plotfig);
   plot(real(eP), imag(eP), 'ko')
   hold on
end



% If the pseudospectrum is unbounded, return Inf
if gamma(k+1) > 0 & (min(svd(P(:,:,k+1))) <= gamma(k+1)*epsln)
    f = inf;
    z = inf;
    return;
end


maxnorm = 0;
for j = 0:k
    maxnorm = max(maxnorm,norm(P(:,:,j+1)));
end

smalltol = 1e-8*max(maxnorm,epsln);

if epsln == 0  % pseudospectrum is just the spectrum
   [sortreal, indx] = sort(-real(eP));
   eP = eP(indx);

   % return pure spectral abscissa
   f = real(eP(1));
   
   % and the eigenvalues with real part equal to the spectral abscissa
   ind = find(real(eP) >= f - smalltol);
   z = eP(ind);
      
else   % much faster than bisection, but just as reliable        
        
   if (epsln < 10^-9)
       % epsilon is too small
       error('poly_pspa: epsilon is too small')
   end
    
   xold = -inf;
   [x,ind] = max(real(eP));  % initial iterate
   y = imag(eP(ind));
   if prtlevel > 0
      fprintf('poly_pspa: x = %22.15f \n', x);
      fprintf('initial estimate is calculated \n');
   end
   iter = 0;
   no_imageig = 0;
   ybest = [];

   % imagtol is used to detect zero real parts
   if eigsolver == 0
       imagtol = smalltol; 
   else
       imagtol = 0;
   end

   while ~no_imageig & x > xold    % x is increasing in exact arithmetic
      iter = iter + 1;
      if iter > 20         
          % too many steps
          error('pspa_poly: too many iterations')
      end
      
      % given current x, look for all relevant intersections of vertical 
      % line with the pseudospectrum, process and return pair midpoints.
      % note: input x is a scalar, but output y is a vector.
      % the input ybest is a scalar.
      [y,imagtol] = poly_pspa_imag(P, epsln, gamma, x, ybest, iter, imagtol, ...
          plotfig, prtlevel, eigsolver);
      
            
      
      if keyb ~= 0
          keyboard
      end
      
      no_imageig = isempty(y);
      if ~no_imageig
         if prtlevel > 0
            fprintf('poly_pspa: y = %22.15f \n', y);    % y may be a vector
         end
         % given the resulting y values, look for rightmost intersection of
         % the corresponding horizontal lines with the pseudospectrum
         % note: input y is a vector, but output x is a scalar: the max
         % of the max values.  ybest is the corresponding y value and is
         % a scalar, even if there was a tie e.g. from a complex conjugate pair.
         xold = x;
         ybestt = ybest;
         [x, ybest] = poly_pspa_real(P, epsln, gamma, y, imagtol, plotfig, xold, eigsolver);

         if x < xold
            x = xold;    % causes while loop to terminate
            ybest = ybestt;

            if prtlevel > 0
               fprintf('poly_pspa: could not find bigger x \n')
            end
         end % end of if x < xold
         
         
         if prtlevel > 0
            fprintf('poly_pspa: x = %22.15f  \n', x);
            fprintf('poly_pspa: iteration %d is completed \n', iter);
         end
      end % end of if ~no_imageig
   end % end of while
   
   
   if isempty(ybest)
       % failed at the first iteration
       error('poly_pspa: failed at the first iteration')
       return;
   end


   f = x;
   z = x+i*ybest;
   if isreal(P) & ~isreal(z)
      z = [z; z'];
   end
   

   
end % end of else


return;