function [f, z] = poly_pspr(P, epsln, gamma, prtlevel, plotfig, keyb)
% Michael Overton and Emre Mengi (Last updated on June 29 2006)
%
% call: function [f, z] = poly_pspr(P, epsln, prtlevel, plotfig, keyb)
% task
%  two-way method to compute the eps-pseudospectral radius of the 
%  polynomial P with respect to the scaling gamma.
%
%  DEFINITION:
%  The epsln-pseudospectral radius of P is the globally optimal value of: 
%      max |z|
%      s.t. sigma_min(P(z) - p_gamma(|z|) I) = epsln (smallest singular value)
%  where p_gamma(x) = sqrt(sum_{j=0}^k gamma_j^2 x^2j.
%
%
%  DETAILS OF THE METHOD:
%  This is an extension of the criss-cross method described in [1].
%  At each iteration the method refines an estimate of the pseudospectral 
%  radius r and a set of midpoints of the intervals in which the circle of 
%  radius r lies inside the pseudospectrum. Initially the estimate is the
%  spectral radius. At each iteration first a radial search from each
%  midpoint returns the largest modulus intersection point of the pseudospectrum 
%  with the ray from the origin going through the midpoint. The estimate
%  is updated to the largest of these values. By the help of a circular
%  search the midpoints of the intervals in which the circle with 
%  radius equal to the new estimate lies inside the pseudospectrum are 
%  computed. The radial search and the circular search are performed by 
%  solving a *-even polynomial eigenvalue problem and by solving a 
%  *-palindromic palindromic eigenvalue problem, respectively (see [2] 
%  for the details).
%
%
%  References
%  1- E.Mengi and M.L.Overton. Algorithms for the computation of the
%     pseudospectral radius and the numerical radius of a matrix. IMA
%     Journal of Numerical Analysis. To appear.
%
%  2- E.Mengi. Measures for robust stability and controllability.
%     Courant Institute of Mathematical Sciences. Ph.D. thesis, 
%     August 2006, in preperation. 
%
%  input
%     P         input matrix polynomial (array of matrices in C^kxnxn)
%     epsln     real >= 0 (0 implies f is spectral radius)
%     gamma     vector of nonnegative real numbers of size k
%               (used for scaling, setting alpha_j=0 blocks the
%               perturbations to K_j)
%     prtlevel  1: chatty; 0: prints only if problems arise (default)
%     plotfig   0: no plot, otherwise figure number for plot
%     keyb      0: no keyboard mode(default), otherwise after each 
%		           iteration enter keyboard mode
%              
%  output
%     f         the epsln-pseudospectral radius of A
%     z         one of the global maximizers except when
%               the matrix is real. If the input matrix is real 
%               it contains either a global optimizer on the
%               real axis or a complex conjugate pair.


if nargin <= 5
    keyb = 0;     % default: don't interact with the user
end
    
if nargin <= 4
    plotfig = 0; % default: don't draw
end

if nargin <= 3
    prtlevel = 0; % default: don't print
end


% error checks
if nargin < 2
    error('poly_pspr: the input polynomial and epsln must be provided')
end

if poly_isnaninf(P) | poly_isnaninf([epsln prtlevel plotfig keyb])
   error('poly_pspr: nan or inf arguments not allowed')
end

if epsln < 0 | imag(epsln) ~= 0
   error('poly_pspr: epsln must be nonnegative real')
end

[nr,nc,kp] = size(P);
if nr ~= nc
   error('poly_pspr: P must be square')
end

% gamma if provided by user must consist of nonnegative real entries
% not all zero
if nargin >= 3
    [gr,gc] = size(gamma);
    
    if gr ~= 1 | gc ~= kp
        error('poly_pspr: gamma must be row vector of size k')
    end
            
    if sum(gamma-abs(gamma)) ~= 0
        error('poly_pspr: gamma must consist of nonnegative real numbers')
    end
    
    if gamma*gamma' == 0
        error('gamma must be nonzero')
    end
end
    

n = nr;
k = kp-1;
% gamma is not provided, set it to the default vector [1 1 ... 1]
if nargin == 2
    gamma = ones(1,k+1);
end

if poly_isnaninf(gamma)
   error('poly_pspr: nan or inf arguments not allowed')
end






% draw the eigenvalues of P
eP = mypolyeig(P);
if plotfig > 0
   figure(plotfig);
   plot(real(eP), imag(eP), 'ko')
   hold on
end

% If the pseudospectrum is unbounded, return Inf
if gamma(k+1) > 0 & (min(svd(P(:,:,k+1))) <= gamma(k+1)*epsln)
    f = inf;
    z = inf;
    return;
end


maxnorm = 0;
for j = 0:k
    maxnorm = max(maxnorm,norm(P(:,:,j+1)));
end

smalltol = 1e-8*max(maxnorm,epsln);

if epsln == 0  % pseudospectrum is just the spectrum
   [sortreal, indx] = sort(-abs(eP));
   eP = eP(indx);
   
   % return pure spectral radius
   f = abs(eP(1));

   % and the eigenvalues with magnitudes equal to the spectral radius
   ind = find(abs(eP) >= f - smalltol);
   z = eP(ind);
   
   
else
    if (epsln < 10^-9)
        warning('epsilon is too small, the result may not be accurate enough')
    end

   rold = -0.00001;
   % initially r is the spectral-radius
   [r,ind] = max(abs(eP));  % initial iterate
   
   if prtlevel > 0
       fprintf('poly_pspr: initially  \n');
       fprintf('poly_pspr: r = %22.15e  \n', r);
   end


   % theta is the angle of an eigenvalue z, s.t |z| = r
   theta = angle(eP(ind));
   thetaold = theta;

   thetabest = [];
   

   
   iter = 0;
   no_thetaeig = 0;
   
   % realtol is used to detect zero real parts upto some tolerance.
   realtol = smalltol;
   % radtol is the tolerance determining how far the eigenvalue magnitudes
   % can be apart from 1 (used by the circular search).
   radtol = smalltol;
   

   while ~no_thetaeig & r > rold    % r is increasing in exact arithmetic

      
      iter = iter + 1;
      
      
      if iter > 20
          error('pspr: too many steps')
      end
      

      thetabestt = thetabest;
      rold = r;

      % given the resulting directions in theta(computed in the
      % previous iteration except when iter=1), look for the circle with
      % the greatest radius intersecting pseudo-spectrum boundary at the
      % given directions.
      % note: input theta is a vector, but output r is a scalar: the max
      % of the max values.  thetabest is the corresponding theta value and is
      % a scalar, even if there was a tie e.g. from a complex conjugate pair.
      [r, thetabest] = poly_pspr_rad(P, gamma, epsln, theta, realtol, ...
                            plotfig, rold, iter);

      % Ensure sigma_min(P(r*e^(i*theta_rho))) > p_gamma(r)*epsilon at the 
      % first interation where r is the estimate of the pseudospectral 
      % radius at the first iteration and theta_rho is the initial radial 
      % search direction.
      if (iter == 1)
          mins = compute_sing(P,gamma,r*exp(i*thetabest));
          ds = sqrt(smalltol);

          beta = epsln - mins;
          incr = abs(beta/ds);
          while (mins < epsln)
              r = r + incr;
              mins = compute_sing(P,gamma,r*exp(i*thetabest));
          end
       
      end
      
      
      if (keyb ~= 0)
           keyboard
      end

      

      
      if r > rold
      	% given current r, look for all relevant intersections of the circle
      	% with radius r with the pseudospectrum, process and return pair midpoints.
      	% note: input r is a scalar, but output theta is a vector.
      	% the input thetabest is a scalar.
      	% there is one difference compared to pseudospectral absicca case
      	% let theta contains t1 <= t2 <= ...  <= tn
      	% the intervals are circular that is we have to check the midpoints
      	% of [t1,t2], [t2,t3], ..., [tn-1,tn],[tn,t1].
      	% In one of these directions there should be a point on the
      	% boundary with a greater radius compared to current maximum radius
        thetaold = theta;        
        theta = poly_pspr_theta(P, gamma, epsln, r, thetabest, iter, ...
                    radtol, smalltol, plotfig, prtlevel);
                
        if keyb ~= 0
            keyboard
        end
      
      	no_thetaeig = isempty(theta);
        
        if (no_thetaeig)
            rold = r;
        end
            
      else
          thetabest = thetabestt;
          r = rold;
      end % end of else
      
      
      if prtlevel > 0
          fprintf('pspr: iteration %d  \n', iter);
          fprintf('pspr: r = %22.15e  \n', r);
          fprintf('pspr: theta= %22.15e  \n', theta);
      end
      
            
   end % end of while

   
      
   
   if isempty(thetabest)
       error('Failed in the first iteration (please set epsln larger)');
   end
   
   
   
   f = r;
   z = r*(cos(thetabest) + i*sin(thetabest));
   
   if isreal(P) & ~isreal(z)
       z = [z; z'];
   end
   
                
end % end of else


return;