function [rbest, thetabest] = poly_pspr_rad(P, gamma, epsln, theta, ...
                            realtol, plotfig, rold, iter)
% Michael Overton and Emre Mengi (Last updated on June 29 2006)
%
% called by poly_pspr.m
%
% Given a set of directions in theta, in each direction finds the point
% on the pseudospectrum boundary with the largest radius, i.e.
%     rnew(j) = max |z|
%    s.t. sigma_min(P(re^(itheta_j))) = p_gamma(r)epsln and angle(z) = theta(j)
% rbest is the maximum of the largest radius values in these direction
%           rbest = max rnew(j)
%                 1<=j<=length(theta)
% thetabest is one of the directions in theta and the angle of a point on 
% the boundary with radius rbest i.e. let rbest = rnew(k), for some k, 
% 1<=k<=length(theta), then thetabest = theta(k).

% no error check here, it is already done in poly_pspr
[n,nc,k] = size(P);
k = k-1;
In = eye(n);
Zn = zeros(n);

for j=1:length(theta)    
    
   thetac = theta(j);
   for l = 0:2*k
       if l == 0
           Q(:,:,l+1) = [-epsln*gamma(l+1)^2*In P(:,:,l+1)'; ...
                         P(:,:,l+1)  -epsln*In];
       else
           if (l <= k)
               if mod(l,2) % l is odd, Qj is skew-Hermitian
                Q(:,:,l+1) = [ Zn (-1)^((l+1)/2)*i*P(:,:,l+1)'*exp(-i*l*thetac); ...
                        (-1)^((l+1)/2)*i*P(:,:,l+1)*exp(i*l*thetac) Zn];
               else % l is even, Qj is Hermitian
                Q(:,:,l+1) = [ epsln*gamma(l/2+1)^2*In*(-1)^(l/2+1) ...
                            (-1)^(l/2)*P(:,:,l+1)'*exp(-i*l*thetac); ...
                            (-1)^(l/2)*P(:,:,l+1)*exp(i*l*thetac) ...
                            Zn];
                end
            else
                if mod(l,2) % l is odd and greater than half of the degree
                 Q(:,:,l+1) = zeros(2*n,2*n);
                else % if l is even and greater than half of the degree
                 Q(:,:,l+1) = [ epsln*gamma(l/2+1)^2*In*(-1)^(l/2+1) Zn; Zn Zn];
                end
            end
       end
   end
   
       
   eQ = mypolyeig(Q);
   

   if min(abs(real(eQ))) <= realtol % check if the polynomial Q has an 
                                    % imaginary eigenvalue
      indx = find(abs(real(eQ)) <= realtol);  % extract such eigenvalues
      rnew(j) = max(imag(eQ(indx)));

      if plotfig > 0
        figure(plotfig)
        % plot a line between old radius and new radius        
        if (rnew(j) > rold)
            plot(rold*cos(theta(j)), rold*sin(theta(j)), 'bx')
            plot([rold*cos(theta(j)) rnew(j)*cos(theta(j))], [rold*sin(theta(j)) rnew(j)*sin(theta(j))], 'm-')
            plot(rnew(j)*cos(theta(j)), rnew(j)*sin(theta(j)), 'b+')
            
            
            if isreal(P) & iter > 1
                plot(rold*cos(theta(j)), -rold*sin(theta(j)), 'bx')
                plot([rold*cos(theta(j)) rnew(j)*cos(theta(j))], -[rold*sin(theta(j)) rnew(j)*sin(theta(j))], 'm-')
                % plot new radius
                plot(rnew(j)*cos(theta(j)), -rnew(j)*sin(theta(j)), 'b+')
            end
        end
                       
      end % end of if plotfig > 0
   else 
      % there should be a point on the boundary of the pseudospectrum
      % in the direction theta(j), because of rounding errors we failed
      % to retrieve it.
      rnew(j) = -inf;
   end % end of else
   
end % end of for

if ~isfinite(max(rnew))
    error('pspr_rad: no intersection point is found by the radial search (please set epsln larger)')
end

% choose the maximum of the largest radii in directions included in theta
[rbest,ind] = max(rnew);
thetabest = theta(ind);

% mark the chosen point on the circle with radius rbest
if plotfig > 0 & rbest > rold
   plot(rbest*cos(thetabest), rbest*sin(thetabest), 'b*')
end
return;