function thetanew = poly_pspr_theta(P, gamma, epsln, r, thetawant, iter, radtol, ...
     smalltol, plotfig, prtlevel)
% Michael Overton and Emre Mengi (Last updated on June 29 2006)
%
% called by poly_pspr.m
%
% Given a radius r, it first computes the intersection points
% of eps-pseudospectrum boundary with the circle of radius r.
% This is achieved by solving a *-palindromic polynomial eigenvalue
% problem of double size and double degree followed by a singular 
% value test. The singular value test is neccessary to eliminate 
% the points for which epsln is a singular value but not the smallest 
% one. Finally the midpoints of two succesive intersection points 
% on the circle with radius r is calculated, i.e. let re^(i*theta_i) 
% and re^(i*theta_(i+1)) be the ith and (i+1)th intersection points, 
% then ith midpoint is re^(i*(theta_i+theta_(i+1))/2). Special attention 
% is paid to keep the angles in the interval [-pi,pi). Furthermore, as 
% it was the case for pseudo-abscissa code, we specifically considered
% the case when the angle from the previous iteration is contained
% in one of the intervals. At the exit thetanew contains the 
% angles of the midpoints where the resolvent norm is less than
% epsilon.

% no error check; the input is assumed to be in correct form
[n,nc,k] = size(P);
k = k-1;

p_gamma = 0;
for j = 0:k
    p_gamma = p_gamma + gamma(j+1)^2*r^(2*j);
end
p_gamma = sqrt(p_gamma);
In = eye(n);
Zn = zeros(n);

for j = 0:2*k
    if mod(j,2)
        if j == k % j is odd and equal to k
            Q(:,:,j+1) = -epsln*p_gamma*eye(2*n);
        else    % j is odd not equal to k
            Q(:,:,j+1) = zeros(2*n);
        end
    else
        if j == k % j is even and equal to k
            Q(:,:,j+1) = [-epsln*p_gamma*In P(:,:,j/2+1)*r^(k/2); ...
                          P(:,:,j/2+1)'*r^(k/2) -epsln*p_gamma*In];
        else % if j is even and not equal to k
            Q(:,:,j+1) = [Zn P(:,:,j/2+1)*r^(j/2); ...
                          P(:,:,k-j/2+1)'*r^(k-j/2) Zn];
        end
    end
end






% solve the palindromic eigenvalue problem
eQ = mypolyeig(Q);

% extract the eigenvalues with magnitude 1
% a small tolerance is used
ind = find((abs(eQ) < (1 + radtol)) & (abs(eQ) > (1 - radtol)));
eQ = eQ(ind);

       
if (isempty(eQ)) % check if Q has an eigenvalue with magnitude 1
   thetanew = [];
else

   % note that if theta is an intersection point e^(i*theta/2) is an 
   % eigenvalue Q
   theta = 2*angle(eQ);
   
   % push the angles into the interval [-pi,pi] 
   for j = 1:length(theta)
       if theta(j) > pi
           theta(j) = theta(j) - 2*pi;
       end
       
       if theta(j) < -pi
           theta(j) = theta(j) + 2*pi;
       end
   end
   % sort the angles
   theta = sort(theta);
          
         
     
   % perform singular value test on the points probably on
   % eps-pseudospectrum boundary.
   % the ones actually on the eps-pseudospectrum are those with smallest 
   % singular value equal to eps
   indx2 = [];
   for j = 1:length(theta)
       if (theta(j) < 0)
           theta(j) = theta(j) + 2*pi;
       end
              
       s = compute_sing(P,gamma,r*exp(i*theta(j)));
       
       if abs(s-epsln) < smalltol
           indx2 = [indx2; j];   % accept this eigenvalue
       end
   end
      
   removed = length(theta) - length(indx2);
   
   if removed > 0
       if prtlevel > 0
           fprintf('poly_pspr_theta: singular value test removed %d eigenvalues \n', removed)
       end
       theta = theta(indx2);
   end
   
   
   if (isempty(theta))
       
       warning('singular value test removed all of the intersection points')
       warning('result may be inccurate')
       thetanew = [];
       
       return;
   end
   
       
   theta = sort(theta);
   
   % organize in pairs and take midpoints
   thetanew = [];
   ind = 0;

   % shift thetawant, the angle from the previous iteration, into
   % the interval [0,2pi]
   if (thetawant < 0)
       thetawant = thetawant + 2*pi;
   end

   for j=1:length(theta)
      thetalow = theta(j);

                        
      if (j < length(theta))
          thetahigh = theta(j+1);
      else
          % the last interval wraps around
          thetahigh = theta(1) + 2*pi;
      end

      
                       
      
          
      % before taking the midpoint, if this interval is not very short,
      % check and see if thetawant is in this interval, well away from the
      % end points.  If so, break this pair into two pairs, one above
      % and one below thetawant, and take midpoints of both.
      inttol = .01 * (thetahigh - thetalow);
      
      % this is needed for the last interval
      if (thetawant+2*pi > thetalow + inttol & ...
          		thetawant+2*pi < thetahigh - inttol)
         thetawantt = thetawant + 2*pi;
      else
         thetawantt = thetawant;
      end
      

      if thetawantt > thetalow + inttol & ...
                thetawantt < thetahigh - inttol
         
            % lower midpoint
            thetamid = (thetalow + thetawantt)/2;
         
            % shift thetanew(ind) into the interval [-pi,pi] again
            if (thetamid >= 2*pi)             
                thetamid = thetamid - 2*pi;
            end         
            if (thetamid >= pi)
                thetamid = thetamid - 2*pi;
            end
         
            % remove the midpoint if the minimum singular value is greater than
            % epsilon, since in this case the midpoint should lie outside the
            % epsilon-pseudospectrum.
            if compute_sing(P,gamma,r*exp(i*thetamid)) <= epsln
                ind = ind + 1;
                thetanew(ind,1) = thetamid;
            end
                  
         
            
            
            % upper midpoint
            thetamid = (thetawantt + thetahigh)/2;
         
            % shift thetanew(ind) into the interval [-pi,pi] again
            if (thetamid >= 2*pi)
                thetamid = thetamid - 2*pi;
            end
            if (thetamid >= pi)
                thetamid = thetamid - 2*pi;
            end
            
            
            % remove the midpoint if the minimum singular value is greater than
            % epsilon, since in this case the midpoint should lie outside the
            % epsilon-pseudospectrum.
            if compute_sing(P,gamma,r*exp(i*thetamid)) <= epsln
                ind = ind + 1;
                thetanew(ind,1) = thetamid;
            end
            
            
     else
         % otherwise, if thetawant is not in the interval
         % take the midpoint of thetalow and thetahigh
         thetamid = (thetalow + thetahigh)/2;   
         
         % shift thetanew(ind) into the interval [-pi,pi] again
         if (thetamid >= 2*pi)
             thetamid = thetamid - 2*pi;
         end
         if (thetamid >= pi)
             thetamid = thetamid - 2*pi;
         end
         
         % remove the midpoint if the minimum singular value is greater than
         % epsilon, since in this case the midpoint should lie outside the
         % epsilon-pseudospectrum.
         if compute_sing(P,gamma,r*exp(i*thetamid)) <= epsln
             ind = ind + 1;
             thetanew(ind,1) = thetamid;
         end

                  
      end % end of else
      
                  
   end  % end of for
   
   
      
   
   
   if plotfig > 0
       % plot the circle with radius r
       plotcircle(r,plotfig);
       figure(plotfig);
      
       % plot the intersection points of the circle (with radius r) 
       % and the pseudo-spectrum
       pointsoncircle = r * (cos(theta) + i*sin(theta));
       plot(real(pointsoncircle), imag(pointsoncircle), 'g+')
       
   end
   
   % if P is real, discard the midpoints in the lower half plane
   if isreal(P)
       indx = find(thetanew >= 0 | thetanew == -pi);
       thetanew = thetanew(indx);
   end

   
end % end of else isempty(eFG)

return;