function [xs,ys] = rect_poly_ps(K,B,alpha,delta1,delta2, ...
                                delta_length,int_length)
% Copyright Emre Mengi, July 2006
% call: rect_poly_ps(K,B,alpha,delta1,delta2,delta_length,int_length)
% Draws the delta-level sets of the function 
%           f(lambda) = sigma_min [K(lambda)/p_alpha(|lambda|)  B]
% for delta = delta_1, delta_1+delta_length, ..., 
% delta_1+floor((delta_2-delta_1)/delta_length)*delta_length.
% The plot is generated by applying the radial search for various
% theta on a 1D grid (see [1] for details). The positive real int_length 
% is the difference between two succesive angles on the grid.
%
% REFERENCES
% [1].  Emre Mengi. On the estimation of the higher order
%       distance to uncontrollability, submitted to SIMAX.

[n,nr,mp] = size(K);

m = mp-1;

if n ~= nr
    error('the input polynomials must be square');
end


realtol = 10^-8;
smalltol = 10^-12;
In = eye(n);
Zn = zeros(n,n);

col = ['b.'; 'g.'; 'r.'; 'c.'; 'm.'; 'k.'; 'y.';...
       'b*'; 'g*'; 'r*'; 'c*'; 'm*'; 'y*'; 'k*'];
l = 1;

for delta = delta1:delta_length:delta2
    
    fprintf('plotting the level set for delta=%f\n',delta);
    Bhat = B*B'/delta-delta*In;
   

    xs = [];
    ys = [];
    for theta = 0:int_length:pi
    
     % for each theta perform a radial search
     
     % Create the matrix polynomial that needs to be solved
     for j = 0:2*m
        if j == 0
            Q(:,:,j+1) = [-delta*alpha(j+1)^2*In K(:,:,j+1)'; ...
                            K(:,:,j+1)  Bhat];
        else
            if (j <= m)
                if mod(j,2) % j is odd, Qj is skew-Hermitian
                    Q(:,:,j+1) = [ Zn (-1)^((j-1)/2)*-i*K(:,:,j+1)'*exp(-i*j*theta); ...
                            (-1)^((j-1)/2)*-i*K(:,:,j+1)*exp(i*j*theta) Zn];
                else % j is even, Qj is Hermitian
                    Q(:,:,j+1) = [ -delta*alpha(j/2+1)^2*In*(-1)^(j/2) ...
                            (-1)^(j/2)*K(:,:,j+1)'*exp(-i*j*theta); ...
                            (-1)^(j/2)*K(:,:,j+1)*exp(i*j*theta) ...
                            Zn];
                end
            else
                if mod(j,2) % j is odd and greater than half of the degree
                    Q(:,:,j+1) = zeros(2*nr,2*nr);
                else % if j is even and greater than half of the degree
                    Q(:,:,j+1) = [ -delta*alpha(j/2+1)^2*In*(-1)^(j/2) Zn; Zn Zn];
                end
            end
        end
     end
        
     % compute the eigenvalues
     eigsQ = mypolyeig(Q);
     % extract the real eigenvalues
     imagind = find(abs(real(eigsQ)) < realtol & eigsQ ~= inf & eigsQ ~= -inf);
     imageigsQ = imag(eigsQ(imagind));
    
    
        % finally test whether the real eigenvalues are the moduli
        % of the intersection points with the level sets
        for j=1:length(imageigsQ)
            mins = compute_rect_sing(K,B,alpha,imageigsQ(j)*exp(i*theta));
            
            % test succeeds: add the point to the set of points
            %                that are going to be plotted.
            if abs(mins-delta) < smalltol
                xs = [xs real(imageigsQ(j)*exp(i*theta))];
                ys = [ys imag(imageigsQ(j)*exp(i*theta))];
            end
        end


    end

     h = plot(xs,ys,col(l,:));
     if ~isempty(xs)
        hold on
        strd(l,:) = sprintf('delta=%.4f',delta);
        l = l+1;
    end
end

if l > 1
    legend(strd)
end

return;