function [test1_test2,z] = perform_new_test(...
                                        A,B,delta1,delta2, ...
                                        print_dtl,upbound,iter)
% Copyright Ming Gu, Emre Mengi, Michael Overton, Jianlin Xia, Jiang Zhu
% auxiliary function called by dist_uncont
% Call : [test1_test2,z] = perform_new_test(A,B,delta1,delta2,print_dtl,upbound,iter) 
% task : checks which one of
%           (*) tau(A,B) <= delta1 or (**) delta2 < tau(A,B)
%        holds using the verification scheme described in [1].
%
%        References: 
%        [1] D. Kressner and E. Mengi, Structure Exploiting Methods
%            for Estimating Robust Stability and Controllability,
%            submitted to CDC2006.
%
% input :
%           A           : nxn complex matrix.
%           B           : nxm complex matrix (m<n).
%           delta1      : the scalar in (*).
%           delta2      : the scalar in (**) (delta2<delta1).
%           print_dtl   : print the details of the progress of the
%                         algorithm or not.
%                         0 : don't print (default).
%                         1 : print.
%           up_bound  	: a scalar such that all points in the 
%                         eps-pseudospectrum of [A B] have
%                         real parts less than up_bound in
%                         absolute value for all eps <= sigma_min[A B].
%           iter        : Number of times this function is called
%                         by the outer function
%
% output :
%           test1_test2 : which one of (*) or (**) holds for the
%                         given delta1 and delta2.
%                         1 : (*) holds.
%                         0 : (**) holds.
%           z           : if (*) holds, this is a point satisfying
%                         sigma_min([A-zI B]) = delta1.
%

[sar,sac] = size(A);
[sbr,sbc] = size(B);



% error checks
if (nargin ~= 7)
    error('Wrong number of input arguments');
end

if (nargout ~= 2)
    error('Wrong number of output arguments');
end


if (sar ~= sac)
    error('A must be a square matrix');
end

if (sar ~= sbr)
    error('A and B must have same number of rows');
end

if (sbc > sbr)
    error('B must have at least as many rows as its columns');
end

if (delta2 > delta1)
    error('The upper bound must be greater than the lower bound');
end                                   


% First set delta, eta.
% If the tests below succeed sigma([A-zI B]) = delta
% for some z meaning delta >= tau(A,B). Hence we set
% delta = delta1.
% If the tests fail, then eta can not be in the interval
% [0,2(delta-tau(A,B))], therefore tau(A,B) > delta-eta/2.
% In other words we want to set delta2 = delta-eta/2 or
% eta=2(delta-delta2).
delta = delta1;
eta = 2*(delta-delta2);


% Bhat appears in the matrix Aalpha for various alpha 
eyen = eye(sar);
Bhat = (B*B')/delta - delta*eyen;


for alpha = -upbound:eta:upbound
    
    realtol = 10^-6;
    smalltol = 10^-8;
    Aalpha = [-(A'-alpha*eyen) delta*eyen; Bhat (A-alpha*eyen)];
    
    eigsAa = eig(Aalpha);

    % extract the imaginary parts of the imaginary
    % eigenvalues of Aalpha.
    indx = find(abs(real(eigsAa)) <= realtol);
    imageigsAa = imag(eigsAa(indx));
    
    for k = 1:length(imageigsAa)
        z0 = alpha + i*imageigsAa(k);
        mins = min(svd([A-z0*eyen B]));
        
        % test succeeded
        if (abs(delta-mins) <= smalltol)
            test1_test2 = 1;
            z = z0;
            return; 
        end
    end
    
end

% test failed
test1_test2 = 0;
z = 0;
return;        
